//*****************************************
// AT mega328P-PU(28pin 300mil SDIP) Four IN-18 Clock Program
// CPU Clock:8MHz
//   iq[Yf[^@WȂBIV[^(8MHz)ɂj
//
// File Name:nixie.c
//
//  2017/09/13 New TAB4 Taishi
//  2017/12/13 Ver 2.00 Taishi
//*****************************************

#include	<avr/pgmspace.h>
#include	<avr/io.h>

#include	"four_in18_clock.h"
#include	"nixie.h"

// **********************************************************
// *********************** NIXE TUBE ************************
// **********************************************************
//

// **********************************************************
//@jLV[ǂ͂O`XƃR\
//@@jLV[ǑI
//@@@ŏʁ@obOfgf
//@@@Qʁ@obPfgf
//@@@Rʁ@obQfgf
//@@@ŉʁ@obRfgf
//@@@R@obSfgf
//
//@@\locSCTCUCV̂Srbĝabcōs
//@@@V@U@T@S
//@@@k@k@k@k@||@O
//@@@k@k@k@g@||@P
//@@@k@k@g@k@||@Q
//@@@k@k@g@g@||@R
//@@@k@g@k@k@||@S
//@@@k@g@k@g@||@T
//@@@k@g@g@k@||@U
//@@@k@g@g@g@||@V
//@@@g@k@k@k@||@W
//@@@g@k@k@g@||@X
//@@@g@k@g@k@||@
//@@@g@k@g@g@||@
//@@@g@g@k@k@||@
//@@@g@g@k@g@||@
//@@@g@g@g@k@||@
//@@@g@g@g@g@||@
//
//@@R\
//@@@ocPFgœ_Akŏ
//
// **********************************************************

unsigned char	path_limit = 1;
unsigned char	change_path = 2;

static	unsigned char	nixie_data[KETA_MAX+1];
static	unsigned char	nixie_dot[KETA_MAX+1];
static	unsigned char	nixie_select[KETA_MAX+1];
static	unsigned char	nixie_act_num = 0;
static	unsigned char	dynamic_nixie = 0;

#define		NIXE_DATA_MASK		0x0d	// PD 1,4--7 Data Mask
#define		NIXE_SELECT_CLR		0xe0	// PC0 -- 4 Clear
#define		NIXE_NUMBER_OFF		0xf0	// PD4--7:'H'
#define		NIXE_NUMBER_MASK	0x0f	// PD4--7:'L'ɂ鎞
#define		NIXE_COLON_ON		0x02	// Colon ON
#define		NIXE_COLON_OFF		0x00	// Colon OFF
//******************************
//  Nixe\@
// *****************************
void	init_nixe(void)
{
	unsigned char	uc;

	PORTC &= NIXE_SELECT_CLR;					// IȂ
	PORTD &= NIXE_DATA_MASK;					// A_}XN
	PORTD |= NIXE_NUMBER_OFF;					// Ȃ
	PORTD &= ~(NIXE_COLON_ON);					// ROFF
	DDRD |= ((1 << DDD1) | (1 << DDD4) | (1 << DDD5) | (1 << DDD6) | (1 << DDD7));	// Port-D Bit1,4567 : Output
	DDRC |= ((1 << DDC0) | (1 << DDC1) | (1 << DDC2) | (1 << DDC3) | (1 << DDC4));	// Port-C Bit01234 : Output

// \A_ f[^
	for(uc = 0;uc <= KETA_MAX; uc++){
		nixie_dot[uc] = NIXE_COLON_OFF;			// _NA
		nixie_data[uc] = NIXE_NUMBER_OFF;		// NA
	}

// I f[^
	nixie_select[0] = 0x01;				// ŏ
	nixie_select[1] = 0x02;				// Q
	nixie_select[2] = 0x04;				// R
	nixie_select[3] = 0x08;				// ŉ
	nixie_select[4] = 0x10;				// R


	nixie_act_num = 0;

	dynamic_nixie = 0;

}
//******************************
// jLV[Ǖ\
// sQ̊荞݂ŌĂ΂
//  d͖TO`Ȃ邪A邭Ȃ
//  ̂O`R͐ SڂRɂ
//******************************
void	proc_dynamic_nixie(void)
{
	static	unsigned char	path=0;
	unsigned char	dd;

	if(dynamic_nixie == 1){
		if(path == path_limit){
			PORTC &= NIXE_SELECT_CLR;						// [0xe0]IȂ
			PORTD |= NIXE_NUMBER_OFF;						// 
			if(nixie_act_num <= (g_keta - 1)){
				PORTD &= ~(NIXE_COLON_ON);						// Rnee
				PORTD &= NIXE_NUMBER_MASK;						// 
				PORTD |= nixie_data[nixie_act_num];				// 
				PORTC |= nixie_select[nixie_act_num];			// I
			}else if(nixie_act_num == g_keta){
				dd = nixie_dot[nixie_act_num];
				if((dd & NIXE_COLON_ON) != 0){					// Rnm̎
					PORTD |= NIXE_COLON_ON;						// Rnm
				}else{
					PORTD &= ~NIXE_COLON_ON;					// Rnee
				}
				PORTC |= nixie_select[nixie_act_num];			// I
			}
			if(g_keta < nixie_act_num){
				nixie_act_num = 0;
			}else{
				nixie_act_num++;
			}
		}else if(path == 0){
			PORTC &= NIXE_SELECT_CLR;						// [0xe0]IȂ
			PORTD |= NIXE_NUMBER_OFF;						// 
			PORTD &= ~(NIXE_COLON_ON);						// Rnee
		}

		path++;
		if(PATH_MAX < path){
			path = 0;
		}

	}
}
//******************************
// jLV[Ǖ\
// sQ̊荞݂ŌĂ΂
//  d͖TO`Ȃ邪A邭Ȃ
//  ̂O`R͐ SڂRɂ
//******************************
void	old_proc_dynamic_nixie(void)
{
	static	unsigned char	path=0;
	unsigned char	dd;

	if(dynamic_nixie == 1){
		PORTC &= NIXE_SELECT_CLR;						// [0xe0]IȂ
		PORTD |= NIXE_NUMBER_OFF;						// 
		if(path == 0){
			if(nixie_act_num <= (g_keta - 1)){
				PORTD &= ~(NIXE_COLON_ON);						// Rnee
				PORTD &= NIXE_NUMBER_MASK;						// 
				PORTD |= nixie_data[nixie_act_num];				// 
				PORTC |= nixie_select[nixie_act_num];			// I
			}else if(nixie_act_num == g_keta){
				dd = nixie_dot[nixie_act_num];
				if((dd & NIXE_COLON_ON) != 0){					// Rnm̎
					PORTD |= NIXE_COLON_ON;						// Rnm
				}else{
					PORTD &= ~NIXE_COLON_ON;					// Rnee
				}
				PORTC |= nixie_select[nixie_act_num];			// I
			}
			if(g_keta < nixie_act_num){
				nixie_act_num = 0;
			}else{
				nixie_act_num++;
			}
		}else{
			PORTD |= NIXE_NUMBER_OFF;						// 
			PORTD &= ~(NIXE_COLON_ON);						// Rnee
		}

		path++;
		if(path_limit <= path){
			path = 0;
		}

	}
}

//******************************
// Dynamic Nixie display Active/Non active
//  on : 1= Active, other=Non Active
// *****************************
void	dynamic_nixie_active(unsigned char on)
{
	dynamic_nixie = on;
}
//******************************
// Nixie \i_Ci~bN_j
//  k : ʒuiO`RAOFŏʁEEERFŉʁj
//  n : O`XAPTiPT͋󔒁j
// *****************************
void	disp_dynamic_nixie_number(unsigned char k, unsigned char n)
{
	if(KETA_MAX <= k){							// IlُȎ
		k = 0;									// Iɍŏʂɂ
	}
	nixie_data[k] = ((n << 4) & 0xf0);
}
//******************************
// R\i_Ci~bN_j
//       d : \iO`PAOF\ȂAPF\j
// *****************************
void	disp_dynamic_colon(unsigned char d)
{
	if(d == 1){
		nixie_dot[4] = NIXE_COLON_ON;
	}else{
		nixie_dot[4] = NIXE_COLON_OFF;
	}
}

//******************************
// Nixie \iX^eBbNN_j
//  k : ʒuiO`RAOFŏʁEEERFŉʁj
//  n : O`XAPTiPT͋󔒁j
//   _Ci~bN\͍sĂȂԂɌ
// *****************************
void	disp_static_nixie_number(unsigned char k, unsigned char n)
{
	PORTC &= NIXE_SELECT_CLR;					// [0xe0]IȂ
	PORTD |= NIXE_NUMBER_OFF;					// 

	if(KETA_MAX <= k){							// IlُȎ
		k = 0;									// Iɍŏʂɂ
	}

	PORTD &= NIXE_NUMBER_MASK;					// 
	PORTD |= ((n << 4) & 0xf0);					// 
	PORTC |= nixie_select[k];					// I

}
//******************************
// R\iX^eBbN_j
//       d : \iO`PAOF\ȂAPF\j
//   _Ci~bN\͍sĂȂԂɌ
// *****************************
void	disp_static_colon(unsigned char d)
{
	PORTC &= NIXE_SELECT_CLR;						// [0xe0]IȂ

	if(d == 1){
		PORTD |= NIXE_COLON_ON;						// Rnm
	}else{
		PORTD &= ~NIXE_COLON_ON;					// Rnee
	}

	PORTC |= nixie_select[4];						// I

}

